<?php
/**
 * This is the entrypoint for the locrMAPS for WooCommerce WordPress plugin.
 *
 * @package locrMAPS for WooCommerce
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMapsForWooCommerce;

use Locr\WordPress\Plugin\LocrMapsForWooCommerce\Admin\CustomProduct;
use Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\ValueObjects\{IconsQuerySelector, SizesQuerySelector};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the plugin.
 */
final class Plugin {
	public const SLUG = 'locrmaps_for_woocommerce';

	/**
	 * This function gets the upload folder.
	 */
	public static function get_upload_folder(): string {
		return wp_upload_dir()['basedir'] . DIRECTORY_SEPARATOR . self::SLUG . '_uploads';
	}

	/**
	 * This function initializes the plugin.
	 */
	public static function init(): void {
		self::init_textdomain();
		self::init_admin_hooks();
		add_action( 'locrmaps_initialized', array( self::class, 'init_public_hooks' ) );
		add_action( 'wp_body_open', array( self::class, 'body_open_hook' ) );
		add_action( 'after_setup_theme', array( self::class, 'after_setup_theme' ) );
		add_action( 'woocommerce_cart_item_removed', array( CustomProduct::class, 'remove_custom_product_image_filename' ), 10, 2 );
	}

	/**
	 * This function is called after the theme is set up.
	 */
	public static function after_setup_theme(): void {
		$options = get_option( self::SLUG . '_options' );
		if ( ! is_array( $options ) ) {
			return;
		}

		if ( isset( $options[ self::SLUG . '_selection_layout' ] ) && is_string( $options[ self::SLUG . '_selection_layout' ] ) ) {
			if ( 'wc-product-property' === $options[ self::SLUG . '_selection_layout' ] ) {
				add_action(
					'woocommerce_variable_add_to_cart',
					function () {
						wp_enqueue_script(
							handle: 'locrmaps-selection-layout',
							src: plugin_dir_url( __DIR__ ) . 'public/js/locrmaps_selection_layout.js',
							ver: md5_file( plugin_dir_path( __DIR__ ) . 'public/js/locrmaps_selection_layout.js' ),
							args: array( 'in_footer' => true ),
						);
						wp_enqueue_style(
							handle: 'locrmaps-selection-layout',
							src: plugin_dir_url( __DIR__ ) . 'public/css/selection-layout-wp-product-property.css',
							ver: md5_file( plugin_dir_path( __DIR__ ) . 'public/css/selection-layout-wp-product-property.css' ),
						);
					}
				);
			}
		}
	}

	/**
	 * This function adds a hidden input field to the body.
	 */
	public static function body_open_hook(): void {
		$options = get_option( self::SLUG . '_options' );
		if ( is_array( $options ) ) {
			if ( isset( $options[ self::SLUG . '_icons_query_selector' ] ) && is_string( $options[ self::SLUG . '_icons_query_selector' ] ) ) {
				print '<input type="hidden" id="locrmaps-' . esc_attr( IconsQuerySelector::ATTRIBUTE_KEY ) . '" name="locrmaps-' . esc_attr( IconsQuerySelector::ATTRIBUTE_KEY ) . '" value="' . esc_attr( $options[ self::SLUG . '_icons_query_selector' ] ) . '"/>';
			}
			if ( isset( $options[ self::SLUG . '_sizes_query_selector' ] ) && is_string( $options[ self::SLUG . '_sizes_query_selector' ] ) ) {
				print '<input type="hidden" id="locrmaps-' . esc_attr( SizesQuerySelector::ATTRIBUTE_KEY ) . '" name="locrmaps-' . esc_attr( SizesQuerySelector::ATTRIBUTE_KEY ) . '" value="' . esc_attr( $options[ self::SLUG . '_sizes_query_selector' ] ) . '"/>';
			}
		}
	}

	/**
	 * This function initializes the admin hooks.
	 */
	private static function init_admin_hooks(): void {
		Admin\SettingsPage::init();
		Admin\CustomProduct::init();
	}

	/**
	 * This function initializes the public hooks.
	 */
	public static function init_public_hooks(): void {
		Frontend\Script::init();
		Frontend\Shortcode::init();
	}

	/**
	 * This function initializes the public hooks.
	 */
	private static function init_textdomain(): void {
		add_action(
			'init',
			function () {
				load_plugin_textdomain( 'locrmaps-for-woocommerce-textdomain', false, dirname( dirname( plugin_basename( __FILE__ ) ) ) . DIRECTORY_SEPARATOR . 'languages' );
			}
		);
		add_filter(
			'load_textdomain_mofile',
			function ( string $mofile, string $domain ) {
				if ( 'locrmaps-for-woocommerce-textdomain' === $domain && false !== strpos( $mofile, WP_LANG_DIR . '/plugins/' ) ) {
					$locale = apply_filters( 'locrmaps_for_woocommerce_locale', determine_locale(), $domain );
					if ( ! is_string( $locale ) ) {
						$locale = determine_locale();
					}
					$mofile = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . dirname( dirname( plugin_basename( __FILE__ ) ) ) . DIRECTORY_SEPARATOR . 'languages' . DIRECTORY_SEPARATOR . $domain . '-' . $locale . '.mo';
				}
				return $mofile;
			},
			10,
			2
		);
	}
}
